from datetime import datetime

from ._OrderState import OrderState


class StateHistory(object):
    def __init__(
        self,           # type: StateHistory
        state,          # type: OrderState
        created_at,     # type: datetime
    ):
        self.state = state
        self.created_at = created_at
