from sysactions import (
    show_messagebox,
    StopAction,
)


def show_nak_message(
    pos_id,         # type: str
    update_key,     # type: str
    message,        # type: str
):
    # type: (...) -> None

    response_message = message.upper()
    if '$' not in response_message:
        response_message = update_key + response_message

    show_messagebox(
        posid=pos_id,
        message=response_message,
        buttons="OK"
    )

    raise StopAction()
