from typing import List


class PartWithDependencies(object):
    def __init__(
        self,           # type: PartWithDependencies
        part_code,      # type: int
        name,           # type: str
        dependencies,   # type: List[int]
    ):
        self.part_code = part_code
        self.name = name
        self.dependencies = dependencies
