class ErrorFormattingReport(Exception):

    def __init__(
        self,
        report_name: str,
        error_message: str,
    ) -> None:
        super(ErrorFormattingReport, self).__init__(f"Error formatting report {report_name}: {error_message}")
