from messagebus import (
    TokenPriority,
    TokenCreator,
)

nreports_group = "6F"


TK_NREPORTS_BUILD_REPORT = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=nreports_group,
    number="1",
)  # 116391937 // 0x6F00001

TK_NREPORTS_REPORT_JSON_PARSER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=nreports_group,
    number="2",
)  # 116391938 // 0x6F00002
