class Part(object):
    def __init__(
        self,
        part_code,      # type: int
        product_name,   # type: str
        item_type,      # type: str
    ):
        # type: (...) -> None

        self.part_code = part_code
        self.product_name = product_name
        self.item_type = item_type
