from typing import Optional


class AsyncTargetDto(object):
    def __init__(
        self,
        reply_subject: str,
        id_field: Optional[str],
    ):
        self.reply_subject = reply_subject
        self.id_field = id_field
