from typing import Any


class OrderItem(object):
    def __init__(
        self,
        name,           # type: unicode
        quantity,       # type: float
        pcode,          # type: int
        unit_price,     # type: float
        context,        # type: str
    ):
        # type: (...) -> None

        self.name = name
        self.quantity = quantity
        self.pcode = pcode
        self.unit_price = unit_price
        self.context = context

    def __getitem__(
        self,
        key,    # type: str
    ):
        # type: (...) -> Any

        return self.__getattribute__(key)
