from messagebus import (
    TokenCreator,
    TokenPriority,
)

TK_CMP_MESSAGE = TokenCreator.create_token(
    priority=TokenPriority.high,
    message_group="3",
    number="6",
)


TABLE_MGR_MESSAGE_GROUP = "C2"

TK_TABLE_MGR_GET_SECTORS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=TABLE_MGR_MESSAGE_GROUP,
    number="1",
)


TK_TABLE_MGR_START_SERVICE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=TABLE_MGR_MESSAGE_GROUP,
    number="2",
)

TK_TABLE_MGR_TABLE_SERVICE_PICTURE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=TABLE_MGR_MESSAGE_GROUP,
    number="3",
)

TK_TABLE_MGR_START_TAB = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=TABLE_MGR_MESSAGE_GROUP,
    number="4",
)

TK_TABLE_MGR_TAB_SERVICE_PICTURE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=TABLE_MGR_MESSAGE_GROUP,
    number="5",
)

CREATE_TAB_FROM_SPLIT_SERVICE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=TABLE_MGR_MESSAGE_GROUP,
    number="6",
)
