from ._TableError import TableError
from typing import Dict


class TabToJoinWithPartialTenders(TableError):

    def __init__(
        self,       # type: TabToJoinWithPartialTenders
        table_id,   # type: str
    ):
        # type: (...) -> None

        message = "$TAB_TO_JOIN_WITH_PARTIAL_TENDERS|{}".format(table_id)
        super(TabToJoinWithPartialTenders, self).__init__(message)

        self.table_id = table_id

    def get_extra_data(
        self,   # type: TabToJoinWithPartialTenders
    ):
        # type: (...) -> Dict[str, str]

        return {
            "tableId": self.table_id,
        }
