from typing import (
    Dict,
    List,
)

from ._TableError import TableError


class MissingAnyOfParameters(TableError):

    def __init__(
        self,           # type: MissingAnyOfParameters
        params=None,    # type: List[str]
    ):
        # type: (...) -> None

        self.params = params

    def get_extra_data(
        self,   # type: MissingAnyOfParameters
    ):
        # type: (...) -> Dict[str, List[str]]

        return {
            "params": self.params,
        }
