from setuptools import setup, find_packages

setup(
    name="PosUpdaterApi",
    version="1.0.0",
    packages=find_packages(where="."),
    install_requires=["MessageBus>=1.1.0"],
    url="https://www.e-deploy.com.br",
    author="Claudinei Nascimento",
    author_email="claudinei.nascimento@e-deploy.com.br",
)
