class GetLoyaltyCustomerInfoRequest(object):

    def __init__(
        self,
        pos_id,         # type: int
        loyalty_id,     # type: str
        order_id,       # type: int
        table_id,       # type: str
        seat,           # type: str
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.loyalty_id = loyalty_id
        self.order_id = order_id
        self.table_id = table_id
        self.seat = seat
