from datetime import date
from typing import Optional

from ._PosState import PosState
from ._User import User


class PosModel(object):
    def __init__(
        self,
        pos_id,                 # type: int
        state,                  # type: PosState
        business_period,        # type: Optional[date]
        logged_user,            # type: Optional[User]
    ):
        self.pos_id = pos_id
        self.state = state
        self.business_period = business_period
        self.logged_user = logged_user
