from datetime import time


class AlertsConfigurations(object):

    def __init__(
        self,                        # type: AlertsConfigurations
        time_calling,                # type: int
        configured_alerts_endpoint,  # type: unicode
        maintenance_window_start,    # type: time
        maintenance_window_end,      # type: time
        days_to_keep_data,           # type: int
    ):
        # type: (...) -> None

        self.time_calling = time_calling
        self.configured_alerts_endpoint = configured_alerts_endpoint
        self.maintenance_window_start = maintenance_window_start
        self.maintenance_window_end = maintenance_window_end
        self.days_to_keep_data = days_to_keep_data
