class ObjectListValueIsNotAList(Exception):

    def __init__(
        self,
        name: str,
    ) -> None:

        self.name = name

    def __str__(self) -> str:
        return f"The objectList {self.name} was not a list"

    def __repr__(self):
        return self.__str__()
