from ._TableError import TableError


class OperatorNotLoggedIn(TableError):
    def __init__(
        self,       # type: OperatorNotLoggedIn
        pos_id,     # type: int
    ):
        self.pos_id = pos_id

    def get_extra_data(self):
        return {
            "posId": self.pos_id,
        }
